"""
Solve Ku = f by Gaussian elimination (no pivoting)

"""
import numpy as np

def mysolve(K, f):
    """    

    Args:
        K (numpy ndarray): square matrix
        f (numpy ndarray): right-hand side vector
            
    Returns:
        numpy ndarray: u, solution to Ku=f

    """
    m,n = K.shape
    assert m == n, "Non-square matrix"
    
    u = np.zeros(n)
    
    # Extended matrix with the right-hand side as last column
    A = np.zeros((n,n+1))
    A[:,0:n] = K
    A[:,n] = f
    
    # Elimination of nonzero elements below the diagonal
    for i in range(n):
        assert A[i,i] != 0.0, "Zero pivot detected"
        
        for j in range(i+1,n):
            Lij = A[j,i] / A[i,i]
        
            for k in range(n+1):
                A[j,k] = A[j,k] - Lij * A[i,k]
    
    # Back substitution
    u[n-1] = A[n-1,n]/A[n-1,n-1]
    
    for i in range(n-2,-1,-1):
        u[i] = A[i,n]
        
        for j in range(i+1,n):
            u[i] = u[i] -  A[i,j]*u[j]
        
            
        u[i] = u[i] / A[i,i]
        
    return u
                
        
            